﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using AGV_WPF.Commands;
using System.Windows.Controls;
using System.Configuration;
using AGV_WPF_Global;
using System.Data.OleDb;
using DataServices;
using System.Data;

namespace AGV_WPF.ExtraUI
{
    public class CardCheckSetWindowModel
    {
        public CardCheckSetWindowModel(Window w)
        {
            checkSetWindow = w as CardCheckSetWindow;
            LoadConfig();
            ModifyCommand = new DelegateCommand();
            ModifyCommand.ExecuteAction = new Action<object>(this.Modify);
            ModifyCommand.CanExecuteFunc = new Func<object, bool>(this.Changed);
            
            ExitCommand = new DelegateCommand();
            ExitCommand.ExecuteAction = new Action<object>(this.Exit);
            CreateDataCommand = new DelegateCommand();
            CreateDataCommand.ExecuteAction = new Action<object>(this.CreateData);
            
        }
        /// <summary>
        /// 关联窗口
        /// </summary>
        CardCheckSetWindow checkSetWindow;
        #region 按钮绑定属性
        public DelegateCommand ModifyCommand { get; set; }
        public DelegateCommand ExitCommand { get; set; }
        public DelegateCommand CreateDataCommand { get; set; }
        
        #endregion

        #region CheckBox绑定属性

        /// <summary>
        /// 读卡检测
        /// </summary>
        private bool isEnableReadCheck;
        public bool IsEnableReadCheck
        {
            get 
            {
                return isEnableReadCheck;
            }
            set 
            {
                if (isEnableReadCheck != value)
                {
                    isEnableReadCheck = value;
                    config.AppSettings.Settings[GlobalPara.strOpenReadCheck].Value = value.ToString();
                    GlobalPara.IsEnableReadCheck = value;
                    IsChanged = true;
                    ModifyCommand.RaiseCanExecuteChanged();
                }
                
            }
        }
        /// <summary>
        /// 漏卡检测
        /// </summary>
        private bool isEnableMissReadCheck;
        public bool IsEnableMissReadCheck
        {
            get
            {
                return isEnableMissReadCheck;
            }
            set
            {
                if (isEnableMissReadCheck != value)
                {
                    isEnableMissReadCheck = value;
                    config.AppSettings.Settings[GlobalPara.strOpenMissReadCheck].Value = value.ToString();
                    GlobalPara.IsEnableMissReadCheck = value;
                    IsChanged = true;
                    ModifyCommand.RaiseCanExecuteChanged();
                }
            }
        }
        /// <summary>
        /// 多读检测
        /// </summary>
        private bool isEnableExtraReadCheck;
        public bool IsEnableExtraReadCheck
        {
            get
            {
                return isEnableExtraReadCheck;
            }
            set
            {
                if (isEnableExtraReadCheck != value)
                {
                    isEnableExtraReadCheck = value; 
                    config.AppSettings.Settings[GlobalPara.strOpenExtraReadCheck].Value = value.ToString();
                    GlobalPara.IsEnableExtraReadCheck = value;
                    IsChanged = true;
                    ModifyCommand.RaiseCanExecuteChanged();
                }
            }
        }
        /// <summary>
        /// 离线检测
        /// </summary>
        private bool isEnableOffLineCheck;
        public bool IsEnableOffLineCheck
        {
            get
            {
                return isEnableOffLineCheck;
            }
            set
            {
                if (isEnableOffLineCheck != value)
                {
                    isEnableOffLineCheck = value;
                    config.AppSettings.Settings[GlobalPara.strOpenOffLineCheck].Value = value.ToString();
                    GlobalPara.IsEnableOffLineCheck = value;
                    IsChanged = true;
                    ModifyCommand.RaiseCanExecuteChanged();
                }
            }
        }
        /// <summary>
        /// 恢复连接检测
        /// </summary>
        private bool isEnableRecoverCheck;
        public bool IsEnableRecoverCheck
        {
            get
            {
                return isEnableRecoverCheck;
            }
            set
            {
                if (isEnableRecoverCheck != value)
                {
                    isEnableRecoverCheck = value;
                    config.AppSettings.Settings[GlobalPara.strOpenRecoverCheck].Value = value.ToString();
                    GlobalPara.IsEnableRecoverCheck = value;
                    IsChanged = true;
                    ModifyCommand.RaiseCanExecuteChanged();
                }
            }
        }
#endregion
        Configuration config = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
        bool IsChanged = false;

        public bool Changed(object parameter)
        {
            return IsChanged;
        }
        public void Modify(object parameter)
        {
            try
            {
                config.Save();
                MessageBox.Show("保存成功");
                checkSetWindow.Close();
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            
        }
        public void Exit(object parameter)
        {
            checkSetWindow.Close();
        }
        public void CreateData(object parameter)
        {
            if (GlobalPara.IsEnableReadCheck)
            {
                MessageBox.Show("请先关闭读卡检测功能");
                return;
            }
            Action<object> import = new Action<object>(ImportHandler);
            window = new Window1();
            window.progressBar1.Value = 0;

            window.textBox1.VerticalScrollBarVisibility = ScrollBarVisibility.Auto;
            window.progressBar1.IsIndeterminate = true;
            import.BeginInvoke(window, null, null);
            window.Show();
        }
        public void LoadConfig()
        {
            isEnableReadCheck = GlobalPara.IsEnableReadCheck;
            isEnableMissReadCheck = GlobalPara.IsEnableMissReadCheck;
            isEnableExtraReadCheck = GlobalPara.IsEnableExtraReadCheck;
            isEnableOffLineCheck = GlobalPara.IsEnableOffLineCheck;
            isEnableRecoverCheck = GlobalPara.IsEnableRecoverCheck;
        }
        Window1 window;
        ExcelDataBase excelDataBase = MainWindow.excelDataBase;
        private void ImportHandler(object o)
        {
            Window1 window = o as Window1;
            try
            {
                if (excelDataBase != null)
                {
                    if (excelDataBase.IsExistTable(GlobalPara.excelFile))
                    {
                        
                        OleDbConnection con = excelDataBase.GetConnection(GlobalPara.excelFile);
                        int temp = excelDataBase.GetTableCount(con, "路线$");
                        if (temp > 0)
                        {
                            if (MessageBox.Show("Excel文件中已有数据，是否覆盖？", "提示", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
                            {
                                try
                                {
                                    excelDataBase.Close();
                                    System.IO.File.Delete(GlobalPara.excelFile);
                                    excelDataBase = new ExcelDataBase(GlobalPara.excelFile);
                                    excelDataBase.CreateTable(GlobalPara.excelFile);
                                }
                                catch (System.Exception ex)
                                {
                                    MessageBox.Show(ex.Message);
                                }

                                Console.WriteLine("删除完成");
                                window.Dispatcher.Invoke(new Action(delegate()
                                {
                                    window.textBox1.AppendText("删除完成\r\n");
                                }));
                            }
                            else
                            {
                                window.Dispatcher.Invoke(new Action(delegate()
                                {
                                    window.Close();
                                }));
                                return;
                            }
                        }
                    }
                    else
                    {
                        excelDataBase.CreateTable(GlobalPara.excelFile);
                        Console.WriteLine("生成表操作");

                        window.Dispatcher.Invoke(new Action(delegate()
                        {
                            window.textBox1.AppendText("生成表操作\r\n");
                        }));
                    }
                }

                DAL.ZSql sqlLine = new DAL.ZSql();
                sqlLine.Open("select distinct LineNum from T_Line order by LineNum");
                DataRowCollection lineCounts = sqlLine.Rows;//路线数量
                sqlLine.Close();
                DAL.ZSql sqlline2 = new DAL.ZSql();
                for (int i = 0; i < lineCounts.Count; i++)
                {
                    Console.WriteLine(string.Format("路线{0}", lineCounts[i]["LineNum"]));

                    window.Dispatcher.Invoke(new Action(delegate()
                    {
                        window.textBox1.AppendText(string.Format("路线{0}\r\n", lineCounts[i]["LineNum"]));
                    }));
                    string s = "select T_Mark.WorkLine,T_Mark.Mark from T_Line left outer join T_Mark on T_Mark.ID = T_Line.MarkID where " + "LineNum=" + lineCounts[i]["LineNum"].ToString() + " and T_Mark.VirtualMark=0 order by MarkOrder";
                    sqlline2.Open(s);
                    DataRowCollection lines = sqlline2.Rows;//路线具体内容
                    for (int j = 0; j < lines.Count; j++)
                    {
                        if (window != null && !window.IsClose)
                        {
                            string str = "insert into [路线$]([生产区号],[AGV编号],[路线编号],[站点序号],[站点编号]) values(" + lines[j]["WorkLine"].ToString() + ",0," + lineCounts[i]["LineNum"].ToString() + "," + (j + 1).ToString() + "," + lines[j]["Mark"].ToString() + ")";
                            if (excelDataBase == null)
                                excelDataBase = new ExcelDataBase(GlobalPara.excelFile);
                            excelDataBase.ExecuteNonQuery(str);


                            window.Dispatcher.Invoke(new Action(delegate()
                            {
                                window.textBox1.AppendText(string.Format("导出数据:生产区{0},AGV编号{1},路线编号{2},站点序号{3},站点编号{4}\r\n", lines[j]["WorkLine"], 0, lineCounts[i]["LineNum"], j + 1, lines[j]["Mark"]));
                                window.textBox1.ScrollToEnd();
                                window.progressBar1.Value += 1;

                            }));
                            Console.WriteLine(string.Format("插入数据:{0}", str));
                        }
                        else
                        {
                            return;
                        }
                    }

                }
                sqlline2.Close();

            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
                return;
            }

            MessageBox.Show("导入成功");
            window.Dispatcher.Invoke(new Action(delegate()
            {
                window.Close();
            }));
        }
    }

}
